@@ -40,25 +40,6 @@
 #include "cogl-journal-private.h"
 #include "cogl-winsys-private.h"
 
-#ifndef HAVE_COGL_GLES2
-
-#define glGenRenderbuffers                ctx->drv.pf_glGenRenderbuffers
-#define glDeleteRenderbuffers             ctx->drv.pf_glDeleteRenderbuffers
-#define glBindRenderbuffer                ctx->drv.pf_glBindRenderbuffer
-#define glRenderbufferStorage             ctx->drv.pf_glRenderbufferStorage
-#define glGenFramebuffers                 ctx->drv.pf_glGenFramebuffers
-#define glBindFramebuffer                 ctx->drv.pf_glBindFramebuffer
-#define glFramebufferTexture2D            ctx->drv.pf_glFramebufferTexture2D
-#define glFramebufferRenderbuffer         ctx->drv.pf_glFramebufferRenderbuffer
-#define glCheckFramebufferStatus          ctx->drv.pf_glCheckFramebufferStatus
-#define glDeleteFramebuffers              ctx->drv.pf_glDeleteFramebuffers
-#define glGetFramebufferAttachmentParameteriv \
-                                          ctx->drv.pf_glGetFramebufferAttachmentParameteriv
-
-#endif
-
-#define glBlitFramebuffer                 ctx->drv.pf_glBlitFramebuffer
-
 #ifndef GL_FRAMEBUFFER
 #define GL_FRAMEBUFFER		0x8D40
 #endif
@@ -167,12 +148,15 @@ _cogl_framebuffer_init (CoglFramebuffer *framebuffer,
   framebuffer->viewport_y       = 0;
   framebuffer->viewport_width   = width;
   framebuffer->viewport_height  = height;
+  framebuffer->dither_enabled   = TRUE;
 
   framebuffer->modelview_stack  = _cogl_matrix_stack_new ();
   framebuffer->projection_stack = _cogl_matrix_stack_new ();
 
   framebuffer->dirty_bitmasks   = TRUE;
 
+  framebuffer->color_mask       = COGL_COLOR_MASK_ALL;
+
   /* Initialise the clip stack */
   _cogl_clip_state_init (&framebuffer->clip_state);
 
@@ -244,18 +228,35 @@ _cogl_framebuffer_get_winsys (CoglFramebuffer *framebuffer)
  * needed when doing operations that may be called whiling flushing
  * the journal */
 void
-_cogl_clear4f (unsigned long buffers,
-               float red,
-               float green,
-               float blue,
-               float alpha)
+_cogl_framebuffer_clear_without_flush4f (CoglFramebuffer *framebuffer,
+                                         unsigned long buffers,
+                                         float red,
+                                         float green,
+                                         float blue,
+                                         float alpha)
 {
   GLbitfield gl_buffers = 0;
 
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
   if (buffers & COGL_BUFFER_BIT_COLOR)
     {
-      GE( glClearColor (red, green, blue, alpha) );
+      GE( ctx, glClearColor (red, green, blue, alpha) );
       gl_buffers |= GL_COLOR_BUFFER_BIT;
+
+      if (ctx->current_gl_color_mask != framebuffer->color_mask)
+        {
+          CoglColorMask color_mask = framebuffer->color_mask;
+          GE( ctx, glColorMask (!!(color_mask & COGL_COLOR_MASK_RED),
+                                !!(color_mask & COGL_COLOR_MASK_GREEN),
+                                !!(color_mask & COGL_COLOR_MASK_BLUE),
+                                !!(color_mask & COGL_COLOR_MASK_ALPHA)));
+          ctx->current_gl_color_mask = color_mask;
+          /* Make sure the ColorMask is updated when the next primitive is drawn */
+          ctx->current_pipeline_changes_since_flush |=
+            COGL_PIPELINE_STATE_LOGIC_OPS;
+          ctx->current_pipeline_age--;
+        }
     }
 
   if (buffers & COGL_BUFFER_BIT_DEPTH)
@@ -277,7 +278,7 @@ _cogl_clear4f (unsigned long buffers,
       return;
     }
 
-  GE (glClear (gl_buffers));
+  GE (ctx, glClear (gl_buffers));
 }
 
 void
@@ -287,12 +288,12 @@ _cogl_framebuffer_dirty (CoglFramebuffer *framebuffer)
 }
 
 void
-_cogl_framebuffer_clear4f (CoglFramebuffer *framebuffer,
-                           unsigned long buffers,
-                           float red,
-                           float green,
-                           float blue,
-                           float alpha)
+cogl_framebuffer_clear4f (CoglFramebuffer *framebuffer,
+                          unsigned long buffers,
+                          float red,
+                          float green,
+                          float blue,
+                          float alpha)
 {
   CoglClipStack *clip_stack = _cogl_framebuffer_get_clip_stack (framebuffer);
   int scissor_x0;
@@ -300,8 +301,6 @@ _cogl_framebuffer_clear4f (CoglFramebuffer *framebuffer,
   int scissor_x1;
   int scissor_y1;
 
-  g_return_if_fail (framebuffer->allocated);
-
   _cogl_clip_stack_get_bounds (clip_stack,
                                &scissor_x0, &scissor_y0,
                                &scissor_x1, &scissor_y1);
@@ -393,7 +392,8 @@ _cogl_framebuffer_clear4f (CoglFramebuffer *framebuffer,
    * always be done first when preparing to draw. */
   _cogl_framebuffer_flush_state (framebuffer, framebuffer, 0);
 
-  _cogl_clear4f (buffers, red, green, blue, alpha);;
+  _cogl_framebuffer_clear_without_flush4f (framebuffer, buffers,
+                                           red, green, blue, alpha);
 
   /* This is a debugging variable used to visually display the quad
    * batches from the journal. It is reset here to increase the
@@ -440,35 +440,21 @@ cleared:
     _cogl_framebuffer_dirty (framebuffer);
 }
 
-/* XXX: We'll need to consider if this API is a good approach for the
- * planned, public, CoglFramebuffer API. A framebuffer may have
- * multiple color buffers associated with it and the user may want to
- * only clear a subset of those buffers. Flags aren't a great
- * mechanism for handling this, but I don't think it would be very
- * convenient if you had to explicitly enumerate the individual
- * ancillary buffers to clear them.
- *
- * My current expectation is that we'll keep this flag based API but
- * also add a way to enumerate the individual color buffers for
- * clearing individually.
- *
- * Note: the 'buffers' and 'color' arguments were switched around on
+/* Note: the 'buffers' and 'color' arguments were switched around on
  * purpose compared to the original cogl_clear API since it was odd
  * that you would be expected to specify a color before even
  * necessarily choosing to clear the color buffer.
  */
 void
-_cogl_framebuffer_clear (CoglFramebuffer *framebuffer,
-                         unsigned long buffers,
-                         const CoglColor *color)
+cogl_framebuffer_clear (CoglFramebuffer *framebuffer,
+                        unsigned long buffers,
+                        const CoglColor *color)
 {
-  g_return_if_fail (framebuffer->allocated);
-
-  _cogl_framebuffer_clear4f (framebuffer, buffers,
-                             cogl_color_get_red_float (color),
-                             cogl_color_get_green_float (color),
-                             cogl_color_get_blue_float (color),
-                             cogl_color_get_alpha_float (color));
+  cogl_framebuffer_clear4f (framebuffer, buffers,
+                            cogl_color_get_red_float (color),
+                            cogl_color_get_green_float (color),
+                            cogl_color_get_blue_float (color),
+                            cogl_color_get_alpha_float (color));
 }
 
 int
@@ -626,52 +612,56 @@ _cogl_framebuffer_flush_dependency_journals (CoglFramebuffer *framebuffer)
 static inline void
 _cogl_framebuffer_init_bits (CoglFramebuffer *framebuffer)
 {
-#ifdef HAVE_COGL_GL
   CoglContext *ctx = framebuffer->context;
-#endif
+
+  cogl_framebuffer_allocate (framebuffer, NULL);
 
   if (G_LIKELY (!framebuffer->dirty_bitmasks))
     return;
 
 #ifdef HAVE_COGL_GL
-  if (cogl_features_available (COGL_FEATURE_OFFSCREEN)
-      && framebuffer->type == COGL_FRAMEBUFFER_TYPE_OFFSCREEN)
+  if (ctx->driver == COGL_DRIVER_GL &&
+      cogl_features_available (COGL_FEATURE_OFFSCREEN) &&
+      framebuffer->type == COGL_FRAMEBUFFER_TYPE_OFFSCREEN)
     {
       GLenum attachment, pname;
 
       attachment = GL_COLOR_ATTACHMENT0;
 
       pname = GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE;
-      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
-                                                 attachment,
-                                                 pname,
-                                                 &framebuffer->red_bits) );
+      GE( ctx, glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                      attachment,
+                                                      pname,
+                                                      &framebuffer->red_bits) );
 
       pname = GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE;
-      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
-                                                 attachment,
-                                                 pname,
-                                                 &framebuffer->green_bits) );
+      GE( ctx, glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                      attachment,
+                                                      pname,
+                                                      &framebuffer->green_bits)
+          );
 
       pname = GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE;
-      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
-                                                 attachment,
-                                                 pname,
-                                                 &framebuffer->blue_bits) );
+      GE( ctx, glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                      attachment,
+                                                      pname,
+                                                      &framebuffer->blue_bits)
+          );
 
       pname = GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE;
-      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
-                                                 attachment,
-                                                 pname,
-                                                 &framebuffer->alpha_bits) );
+      GE( ctx, glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                      attachment,
+                                                      pname,
+                                                      &framebuffer->alpha_bits)
+          );
     }
   else
 #endif /* HAVE_COGL_GL */
     {
-      GE( glGetIntegerv (GL_RED_BITS,   &framebuffer->red_bits)   );
-      GE( glGetIntegerv (GL_GREEN_BITS, &framebuffer->green_bits) );
-      GE( glGetIntegerv (GL_BLUE_BITS,  &framebuffer->blue_bits)  );
-      GE( glGetIntegerv (GL_ALPHA_BITS, &framebuffer->alpha_bits) );
+      GE( ctx, glGetIntegerv (GL_RED_BITS,   &framebuffer->red_bits)   );
+      GE( ctx, glGetIntegerv (GL_GREEN_BITS, &framebuffer->green_bits) );
+      GE( ctx, glGetIntegerv (GL_BLUE_BITS,  &framebuffer->blue_bits)  );
+      GE( ctx, glGetIntegerv (GL_ALPHA_BITS, &framebuffer->alpha_bits) );
     }
 
 
@@ -729,28 +719,30 @@ try_creating_fbo (CoglOffscreen *offscreen,
   ctx->dirty_bound_framebuffer = 1;
 
   /* Generate framebuffer */
-  glGenFramebuffers (1, &fbo_gl_handle);
-  GE (glBindFramebuffer (GL_FRAMEBUFFER, fbo_gl_handle));
+  ctx->glGenFramebuffers (1, &fbo_gl_handle);
+  GE (ctx, glBindFramebuffer (GL_FRAMEBUFFER, fbo_gl_handle));
   offscreen->fbo_handle = fbo_gl_handle;
 
-  GE (glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-                              tex_gl_target, tex_gl_handle, data->level));
+  GE (ctx, glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
+                                   tex_gl_target, tex_gl_handle, data->level));
 
   if (flags & _TRY_DEPTH_STENCIL)
     {
       /* Create a renderbuffer for depth and stenciling */
-      GE (glGenRenderbuffers (1, &gl_depth_stencil_handle));
-      GE (glBindRenderbuffer (GL_RENDERBUFFER, gl_depth_stencil_handle));
-      GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_STENCIL,
-                                 data->level_width,
-                                 data->level_height));
-      GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
-      GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
-                                     GL_STENCIL_ATTACHMENT,
-                                     GL_RENDERBUFFER, gl_depth_stencil_handle));
-      GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
-                                     GL_DEPTH_ATTACHMENT,
-                                     GL_RENDERBUFFER, gl_depth_stencil_handle));
+      GE (ctx, glGenRenderbuffers (1, &gl_depth_stencil_handle));
+      GE (ctx, glBindRenderbuffer (GL_RENDERBUFFER, gl_depth_stencil_handle));
+      GE (ctx, glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_STENCIL,
+                                      data->level_width,
+                                      data->level_height));
+      GE (ctx, glBindRenderbuffer (GL_RENDERBUFFER, 0));
+      GE (ctx, glFramebufferRenderbuffer (GL_FRAMEBUFFER,
+                                          GL_STENCIL_ATTACHMENT,
+                                          GL_RENDERBUFFER,
+                                          gl_depth_stencil_handle));
+      GE (ctx, glFramebufferRenderbuffer (GL_FRAMEBUFFER,
+                                          GL_DEPTH_ATTACHMENT,
+                                          GL_RENDERBUFFER,
+                                          gl_depth_stencil_handle));
       offscreen->renderbuffers =
         g_slist_prepend (offscreen->renderbuffers,
                          GUINT_TO_POINTER (gl_depth_stencil_handle));
@@ -758,15 +750,15 @@ try_creating_fbo (CoglOffscreen *offscreen,
 
   if (flags & _TRY_DEPTH)
     {
-      GE (glGenRenderbuffers (1, &gl_depth_handle));
-      GE (glBindRenderbuffer (GL_RENDERBUFFER, gl_depth_handle));
+      GE (ctx, glGenRenderbuffers (1, &gl_depth_handle));
+      GE (ctx, glBindRenderbuffer (GL_RENDERBUFFER, gl_depth_handle));
       /* For now we just ask for GL_DEPTH_COMPONENT16 since this is all that's
        * available under GLES */
-      GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_COMPONENT16,
+      GE (ctx, glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_COMPONENT16,
                                  data->level_width,
                                  data->level_height));
-      GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
-      GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
+      GE (ctx, glBindRenderbuffer (GL_RENDERBUFFER, 0));
+      GE (ctx, glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_DEPTH_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_depth_handle));
       offscreen->renderbuffers =
@@ -776,13 +768,13 @@ try_creating_fbo (CoglOffscreen *offscreen,
 
   if (flags & _TRY_STENCIL)
     {
-      GE (glGenRenderbuffers (1, &gl_stencil_handle));
-      GE (glBindRenderbuffer (GL_RENDERBUFFER, gl_stencil_handle));
-      GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_STENCIL_INDEX8,
+      GE (ctx, glGenRenderbuffers (1, &gl_stencil_handle));
+      GE (ctx, glBindRenderbuffer (GL_RENDERBUFFER, gl_stencil_handle));
+      GE (ctx, glRenderbufferStorage (GL_RENDERBUFFER, GL_STENCIL_INDEX8,
                                  data->level_width,
                                  data->level_height));
-      GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
-      GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
+      GE (ctx, glBindRenderbuffer (GL_RENDERBUFFER, 0));
+      GE (ctx, glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_STENCIL_ATTACHMENT,
                                      GL_RENDERBUFFER, gl_stencil_handle));
       offscreen->renderbuffers =
@@ -791,18 +783,18 @@ try_creating_fbo (CoglOffscreen *offscreen,
     }
 
   /* Make sure it's complete */
-  status = glCheckFramebufferStatus (GL_FRAMEBUFFER);
+  status = ctx->glCheckFramebufferStatus (GL_FRAMEBUFFER);
 
   if (status != GL_FRAMEBUFFER_COMPLETE)
     {
       GSList *l;
 
-      GE (glDeleteFramebuffers (1, &fbo_gl_handle));
+      GE (ctx, glDeleteFramebuffers (1, &fbo_gl_handle));
 
       for (l = offscreen->renderbuffers; l; l = l->next)
         {
           GLuint renderbuffer = GPOINTER_TO_UINT (l->data);
-          GE (glDeleteRenderbuffers (1, &renderbuffer));
+          GE (ctx, glDeleteRenderbuffers (1, &renderbuffer));
         }
 
       g_slist_free (offscreen->renderbuffers);
@@ -826,6 +818,8 @@ _cogl_offscreen_new_to_texture_full (CoglHandle texhandle,
   CoglFramebufferTryFBOData data;
   gboolean            fbo_created;
 
+  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+
   if (!cogl_features_available (COGL_FEATURE_OFFSCREEN))
     return COGL_INVALID_HANDLE;
 
@@ -880,7 +874,8 @@ _cogl_offscreen_new_to_texture_full (CoglHandle texhandle,
       if ((have_working_flags &&
            try_creating_fbo (offscreen, flags, &data)) ||
 #ifdef HAVE_COGL_GL
-          try_creating_fbo (offscreen, flags = _TRY_DEPTH_STENCIL, &data) ||
+          (ctx->driver == COGL_DRIVER_GL &&
+           try_creating_fbo (offscreen, flags = _TRY_DEPTH_STENCIL, &data)) ||
 #endif
           try_creating_fbo (offscreen, flags = _TRY_DEPTH | _TRY_STENCIL,
                             &data) ||
@@ -941,9 +936,7 @@ static void
 _cogl_offscreen_free (CoglOffscreen *offscreen)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (offscreen);
-#ifndef HAVE_COGL_GLES2
   CoglContext *ctx = framebuffer->context;
-#endif
   GSList *l;
 
   /* Chain up to parent */
@@ -952,11 +945,11 @@ _cogl_offscreen_free (CoglOffscreen *offscreen)
   for (l = offscreen->renderbuffers; l; l = l->next)
     {
       GLuint renderbuffer = GPOINTER_TO_UINT (l->data);
-      GE (glDeleteRenderbuffers (1, &renderbuffer));
+      GE (ctx, glDeleteRenderbuffers (1, &renderbuffer));
     }
   g_slist_free (offscreen->renderbuffers);
 
-  GE (glDeleteFramebuffers (1, &offscreen->fbo_handle));
+  GE (ctx, glDeleteFramebuffers (1, &offscreen->fbo_handle));
 
   if (offscreen->texture != COGL_INVALID_HANDLE)
     cogl_handle_unref (offscreen->texture);
@@ -1368,7 +1361,7 @@ bind_gl_framebuffer (CoglContext *ctx,
                      CoglFramebuffer *framebuffer)
 {
   if (framebuffer->type == COGL_FRAMEBUFFER_TYPE_OFFSCREEN)
-    GE (glBindFramebuffer (target,
+    GE (ctx, glBindFramebuffer (target,
                            COGL_OFFSCREEN (framebuffer)->fbo_handle));
   else
     {
@@ -1377,7 +1370,7 @@ bind_gl_framebuffer (CoglContext *ctx,
       winsys->onscreen_bind (COGL_ONSCREEN (framebuffer));
       /* glBindFramebuffer is an an extension with OpenGL ES 1.1 */
       if (cogl_features_available (COGL_FEATURE_OFFSCREEN))
-        GE (glBindFramebuffer (target, 0));
+        GE (ctx, glBindFramebuffer (target, 0));
     }
 }
 
@@ -1388,6 +1381,10 @@ _cogl_framebuffer_flush_state (CoglFramebuffer *draw_buffer,
 {
   CoglContext *ctx = draw_buffer->context;
 
+  /* Lazily ensure the framebuffer has been allocated */
+  cogl_framebuffer_allocate (draw_buffer, NULL);
+  cogl_framebuffer_allocate (read_buffer, NULL);
+
   if (ctx->dirty_bound_framebuffer)
     {
       if (draw_buffer == read_buffer)
@@ -1435,24 +1432,28 @@ _cogl_framebuffer_flush_state (CoglFramebuffer *draw_buffer,
                  draw_buffer->viewport_width,
                  draw_buffer->viewport_height);
 
-      GE (glViewport (draw_buffer->viewport_x,
+      GE (ctx, glViewport (draw_buffer->viewport_x,
                       gl_viewport_y,
                       draw_buffer->viewport_width,
                       draw_buffer->viewport_height));
       ctx->dirty_gl_viewport = FALSE;
     }
 
-  /* since we might have changed the framebuffer, we should initialize
-   * the bits; this is a no-op if they have already been initialized
-   */
-  _cogl_framebuffer_init_bits (draw_buffer);
-  _cogl_framebuffer_init_bits (read_buffer);
+  if (ctx->current_gl_dither_enabled != draw_buffer->dither_enabled)
+    {
+      if (draw_buffer->dither_enabled)
+        GE (ctx, glEnable (GL_DITHER));
+      else
+        GE (ctx, glDisable (GL_DITHER));
+      ctx->current_gl_dither_enabled = draw_buffer->dither_enabled;
+    }
 
   /* XXX: Flushing clip state may trash the modelview and projection
    * matrices so we must do it before flushing the matrices...
    */
   if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_CLIP_STATE))
-    _cogl_clip_state_flush (&draw_buffer->clip_state);
+    _cogl_clip_state_flush (&draw_buffer->clip_state,
+                            draw_buffer);
 
   if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_MODELVIEW))
     _cogl_matrix_stack_flush_to_gl (draw_buffer->modelview_stack,
@@ -1463,7 +1464,7 @@ _cogl_framebuffer_flush_state (CoglFramebuffer *draw_buffer,
 }
 
 int
-_cogl_framebuffer_get_red_bits (CoglFramebuffer *framebuffer)
+cogl_framebuffer_get_red_bits (CoglFramebuffer *framebuffer)
 {
   _cogl_framebuffer_init_bits (framebuffer);
 
@@ -1471,7 +1472,7 @@ _cogl_framebuffer_get_red_bits (CoglFramebuffer *framebuffer)
 }
 
 int
-_cogl_framebuffer_get_green_bits (CoglFramebuffer *framebuffer)
+cogl_framebuffer_get_green_bits (CoglFramebuffer *framebuffer)
 {
   _cogl_framebuffer_init_bits (framebuffer);
 
@@ -1479,7 +1480,7 @@ _cogl_framebuffer_get_green_bits (CoglFramebuffer *framebuffer)
 }
 
 int
-_cogl_framebuffer_get_blue_bits (CoglFramebuffer *framebuffer)
+cogl_framebuffer_get_blue_bits (CoglFramebuffer *framebuffer)
 {
   _cogl_framebuffer_init_bits (framebuffer);
 
@@ -1487,13 +1488,65 @@ _cogl_framebuffer_get_blue_bits (CoglFramebuffer *framebuffer)
 }
 
 int
-_cogl_framebuffer_get_alpha_bits (CoglFramebuffer *framebuffer)
+cogl_framebuffer_get_alpha_bits (CoglFramebuffer *framebuffer)
 {
   _cogl_framebuffer_init_bits (framebuffer);
 
   return framebuffer->alpha_bits;
 }
 
+CoglColorMask
+cogl_framebuffer_get_color_mask (CoglFramebuffer *framebuffer)
+{
+  return framebuffer->color_mask;
+}
+
+void
+cogl_framebuffer_set_color_mask (CoglFramebuffer *framebuffer,
+                                 CoglColorMask color_mask)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  cogl_flush (); /* XXX: Currently color mask changes don't go through the journal */
+  framebuffer->color_mask = color_mask;
+
+  /* Make sure the ColorMask is updated when the next primitive is drawn */
+  ctx->current_pipeline_changes_since_flush |=
+    COGL_PIPELINE_STATE_LOGIC_OPS;
+  ctx->current_pipeline_age--;
+}
+
+gboolean
+cogl_framebuffer_get_dither_enabled (CoglFramebuffer *framebuffer)
+{
+  return framebuffer->dither_enabled;
+}
+
+void
+cogl_framebuffer_set_dither_enabled (CoglFramebuffer *framebuffer,
+                                     gboolean dither_enabled)
+{
+  if (framebuffer->dither_enabled == dither_enabled)
+    return;
+
+  cogl_flush (); /* Currently dithering changes aren't tracked in the journal */
+  framebuffer->dither_enabled = dither_enabled;
+}
+
+CoglPixelFormat
+cogl_framebuffer_get_color_format (CoglFramebuffer *framebuffer)
+{
+  return framebuffer->format;
+}
+
+CoglContext *
+cogl_framebuffer_get_context (CoglFramebuffer *framebuffer)
+{
+  g_return_val_if_fail (framebuffer != NULL, NULL);
+
+  return framebuffer->context;
+}
+
 gboolean
 _cogl_framebuffer_try_fast_read_pixel (CoglFramebuffer *framebuffer,
                                        int x,
@@ -1594,9 +1647,9 @@ _cogl_blit_framebuffer (unsigned int src_x,
      by the scissor and we want to hide this feature for the Cogl API
      because it's not obvious to an app how the clip state will affect
      the scissor */
-  _cogl_clip_stack_flush (NULL);
+  _cogl_clip_stack_flush (NULL, draw_buffer);
 
-  glBlitFramebuffer (src_x, src_y,
+  ctx->glBlitFramebuffer (src_x, src_y,
                      src_x + width, src_y + height,
                      dst_x, dst_y,
                      dst_x + width, dst_y + height,
@@ -1619,7 +1672,7 @@ cogl_framebuffer_swap_buffers (CoglFramebuffer *framebuffer)
 
 void
 cogl_framebuffer_swap_region (CoglFramebuffer *framebuffer,
-                              int *rectangles,
+                              const int *rectangles,
                               int n_rectangles)
 {
   /* FIXME: we shouldn't need to flush *all* journals here! */
@@ -1641,7 +1694,7 @@ cogl_framebuffer_swap_region (CoglFramebuffer *framebuffer,
 
 #ifdef COGL_HAS_X11_SUPPORT
 void
-cogl_onscreen_x11_set_foreign_window_xid (CoglOnscreen *onscreen,
+cogl_x11_onscreen_set_foreign_window_xid (CoglOnscreen *onscreen,
                                           guint32 xid,
                                           CoglOnscreenX11MaskCallback update,
                                           void *user_data)
@@ -1656,7 +1709,7 @@ cogl_onscreen_x11_set_foreign_window_xid (CoglOnscreen *onscreen,
 }
 
 guint32
-cogl_onscreen_x11_get_window_xid (CoglOnscreen *onscreen)
+cogl_x11_onscreen_get_window_xid (CoglOnscreen *onscreen)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
 
@@ -1674,7 +1727,7 @@ cogl_onscreen_x11_get_window_xid (CoglOnscreen *onscreen)
 }
 
 guint32
-cogl_onscreen_x11_get_visual_xid (CoglOnscreen *onscreen)
+cogl_x11_onscreen_get_visual_xid (CoglOnscreen *onscreen)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
   const CoglWinsysVtable *winsys = _cogl_framebuffer_get_winsys (framebuffer);
@@ -1695,14 +1748,14 @@ cogl_onscreen_x11_get_visual_xid (CoglOnscreen *onscreen)
 #ifdef COGL_HAS_WIN32_SUPPORT
 
 void
-cogl_onscreen_win32_set_foreign_window (CoglOnscreen *onscreen,
+cogl_win32_onscreen_set_foreign_window (CoglOnscreen *onscreen,
                                         HWND hwnd)
 {
   onscreen->foreign_hwnd = hwnd;
 }
 
 HWND
-cogl_onscreen_win32_get_window (CoglOnscreen *onscreen)
+cogl_win32_onscreen_get_window (CoglOnscreen *onscreen)
 {
   if (onscreen->foreign_hwnd)
     return onscreen->foreign_hwnd;
