@@ -49,22 +49,6 @@
  * GL/GLES compatability defines for pipeline thingies:
  */
 
-#ifdef HAVE_COGL_GL
-#define glActiveTexture ctx->drv.pf_glActiveTexture
-#define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
-#define glBlendFuncSeparate ctx->drv.pf_glBlendFuncSeparate
-#define glBlendEquation ctx->drv.pf_glBlendEquation
-#define glBlendColor ctx->drv.pf_glBlendColor
-#define glBlendEquationSeparate ctx->drv.pf_glBlendEquationSeparate
-
-#define glProgramString ctx->drv.pf_glProgramString
-#define glBindProgram ctx->drv.pf_glBindProgram
-#define glDeletePrograms ctx->drv.pf_glDeletePrograms
-#define glGenPrograms ctx->drv.pf_glGenPrograms
-#define glProgramLocalParameter4fv ctx->drv.pf_glProgramLocalParameter4fv
-#define glUseProgram ctx->drv.pf_glUseProgram
-#endif
-
 /* These aren't defined in the GLES headers */
 #ifndef GL_POINT_SPRITE
 #define GL_POINT_SPRITE 0x8861
@@ -146,7 +130,7 @@ _cogl_set_active_texture_unit (int unit_index)
 
   if (ctx->active_texture_unit != unit_index)
     {
-      GE (glActiveTexture (GL_TEXTURE0 + unit_index));
+      GE (ctx, glActiveTexture (GL_TEXTURE0 + unit_index));
       ctx->active_texture_unit = unit_index;
     }
 }
@@ -198,7 +182,7 @@ _cogl_bind_gl_texture_transient (GLenum gl_target,
       !unit->is_foreign)
     return;
 
-  GE (glBindTexture (gl_target, gl_texture));
+  GE (ctx, glBindTexture (gl_target, gl_texture));
 
   unit->dirty_gl_texture = TRUE;
   unit->is_foreign = is_foreign;
@@ -224,7 +208,7 @@ _cogl_delete_gl_texture (GLuint gl_texture)
         }
     }
 
-  GE (glDeleteTextures (1, &gl_texture));
+  GE (ctx, glDeleteTextures (1, &gl_texture));
 }
 
 /* Whenever the underlying GL texture storage of a CoglTexture is
@@ -256,31 +240,23 @@ _cogl_pipeline_texture_storage_change_notify (CoglHandle texture)
 static void
 set_glsl_program (GLuint gl_program)
 {
-#ifdef HAVE_COGL_GLES
-
-  g_return_if_reached ();
-
-#else /* HAVE_COGL_GLES */
-
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   if (ctx->current_gl_program != gl_program)
     {
       GLenum gl_error;
 
-      while ((gl_error = glGetError ()) != GL_NO_ERROR)
+      while ((gl_error = ctx->glGetError ()) != GL_NO_ERROR)
         ;
-      glUseProgram (gl_program);
-      if (glGetError () == GL_NO_ERROR)
+      ctx->glUseProgram (gl_program);
+      if (ctx->glGetError () == GL_NO_ERROR)
         ctx->current_gl_program = gl_program;
       else
         {
-          GE( glUseProgram (0) );
+          GE( ctx, glUseProgram (0) );
           ctx->current_gl_program = 0;
         }
     }
-
-#endif /* HAVE_COGL_GLES */
 }
 
 void
@@ -304,7 +280,7 @@ _cogl_use_fragment_program (GLuint gl_program, CoglPipelineProgramType type)
 
         case COGL_PIPELINE_PROGRAM_TYPE_ARBFP:
 #ifdef HAVE_COGL_GL
-          GE( glDisable (GL_FRAGMENT_PROGRAM_ARB) );
+          GE( ctx, glDisable (GL_FRAGMENT_PROGRAM_ARB) );
 #endif
           break;
 
@@ -318,7 +294,7 @@ _cogl_use_fragment_program (GLuint gl_program, CoglPipelineProgramType type)
         {
         case COGL_PIPELINE_PROGRAM_TYPE_ARBFP:
 #ifdef HAVE_COGL_GL
-          GE( glEnable (GL_FRAGMENT_PROGRAM_ARB) );
+          GE( ctx, glEnable (GL_FRAGMENT_PROGRAM_ARB) );
 #endif
           break;
 
@@ -411,27 +387,7 @@ _cogl_use_vertex_program (GLuint gl_program, CoglPipelineProgramType type)
   ctx->current_vertex_program_type = type;
 }
 
-#if defined (COGL_PIPELINE_FRAGEND_GLSL) || \
-    defined (COGL_PIPELINE_FRAGEND_ARBFP)
-int
-_cogl_get_max_texture_image_units (void)
-{
-  _COGL_GET_CONTEXT (ctx, 0);
-
-  /* This function is called quite often so we cache the value to
-     avoid too many GL calls */
-  if (G_UNLIKELY (ctx->max_texture_image_units == -1))
-    {
-      ctx->max_texture_image_units = 1;
-      GE (glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS,
-                         &ctx->max_texture_image_units));
-    }
-
-  return ctx->max_texture_image_units;
-}
-#endif
-
-#ifndef HAVE_COGL_GLES
+#if defined(HAVE_COGL_GLES2) || defined(HAVE_COGL_GL)
 
 static gboolean
 blend_factor_uses_constant (GLenum blend_factor)
@@ -451,32 +407,31 @@ flush_depth_state (CoglDepthState *depth_state)
 
   if (ctx->depth_test_function_cache != depth_state->test_function)
     {
-      GE (glDepthFunc (depth_state->test_function));
+      GE (ctx, glDepthFunc (depth_state->test_function));
       ctx->depth_test_function_cache = depth_state->test_function;
     }
 
   if (ctx->depth_writing_enabled_cache != depth_state->write_enabled)
     {
-      GE (glDepthMask (depth_state->write_enabled ?
-                       GL_TRUE : GL_FALSE));
+      GE (ctx, glDepthMask (depth_state->write_enabled ?
+                            GL_TRUE : GL_FALSE));
       ctx->depth_writing_enabled_cache = depth_state->write_enabled;
     }
 
-#ifndef COGL_HAS_GLES
-  if (ctx->depth_range_near_cache != depth_state->range_near ||
-      ctx->depth_range_far_cache != depth_state->range_far)
+  if (ctx->driver != COGL_DRIVER_GLES1 &&
+      (ctx->depth_range_near_cache != depth_state->range_near ||
+       ctx->depth_range_far_cache != depth_state->range_far))
     {
-#ifdef COGL_HAS_GLES2
-      GE (glDepthRangef (depth_state->range_near,
-                         depth_state->range_far));
-#else
-      GE (glDepthRange (depth_state->range_near,
-                        depth_state->range_far));
-#endif
+      if (ctx->driver == COGL_DRIVER_GLES2)
+        GE (ctx, glDepthRangef (depth_state->range_near,
+                                depth_state->range_far));
+      else
+        GE (ctx, glDepthRange (depth_state->range_near,
+                               depth_state->range_far));
+
       ctx->depth_range_near_cache = depth_state->range_near;
       ctx->depth_range_far_cache = depth_state->range_far;
     }
-#endif /* COGL_HAS_GLES */
 }
 
 static void
@@ -488,8 +443,8 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* On GLES2 we'll flush the color later */
-#ifndef HAVE_COGL_GLES2
-  if (!skip_gl_color)
+  if (ctx->driver != COGL_DRIVER_GLES2 &&
+      !skip_gl_color)
     {
       if ((pipelines_difference & COGL_PIPELINE_STATE_COLOR) ||
           /* Assume if we were previously told to skip the color, then
@@ -498,13 +453,12 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
         {
           CoglPipeline *authority =
             _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_COLOR);
-          GE (glColor4ub (cogl_color_get_red_byte (&authority->color),
-                          cogl_color_get_green_byte (&authority->color),
-                          cogl_color_get_blue_byte (&authority->color),
-                          cogl_color_get_alpha_byte (&authority->color)));
+          GE (ctx, glColor4ub (cogl_color_get_red_byte (&authority->color),
+                               cogl_color_get_green_byte (&authority->color),
+                               cogl_color_get_blue_byte (&authority->color),
+                               cogl_color_get_alpha_byte (&authority->color)));
         }
     }
-#endif
 
   if (pipelines_difference & COGL_PIPELINE_STATE_BLEND)
     {
@@ -513,97 +467,103 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
       CoglPipelineBlendState *blend_state =
         &authority->big_state->blend_state;
 
-#if defined (HAVE_COGL_GLES2)
-      gboolean have_blend_equation_seperate = TRUE;
-      gboolean have_blend_func_separate = TRUE;
-#elif defined (HAVE_COGL_GL)
-      gboolean have_blend_equation_seperate = FALSE;
-      gboolean have_blend_func_separate = FALSE;
-      if (ctx->drv.pf_glBlendEquationSeparate) /* Only GL 2.0 + */
-        have_blend_equation_seperate = TRUE;
-      if (ctx->drv.pf_glBlendFuncSeparate) /* Only GL 1.4 + */
-        have_blend_func_separate = TRUE;
-#endif
-
-#ifndef HAVE_COGL_GLES /* GLES 1 only has glBlendFunc */
-      if (blend_factor_uses_constant (blend_state->blend_src_factor_rgb) ||
-          blend_factor_uses_constant (blend_state->blend_src_factor_alpha) ||
-          blend_factor_uses_constant (blend_state->blend_dst_factor_rgb) ||
-          blend_factor_uses_constant (blend_state->blend_dst_factor_alpha))
+      /* GLES 1 only has glBlendFunc */
+      if (ctx->driver == COGL_DRIVER_GLES1)
         {
-          float red =
-            cogl_color_get_red_float (&blend_state->blend_constant);
-          float green =
-            cogl_color_get_green_float (&blend_state->blend_constant);
-          float blue =
-            cogl_color_get_blue_float (&blend_state->blend_constant);
-          float alpha =
-            cogl_color_get_alpha_float (&blend_state->blend_constant);
+          GE (ctx, glBlendFunc (blend_state->blend_src_factor_rgb,
+                                blend_state->blend_dst_factor_rgb));
+        }
+#if defined(HAVE_COGL_GLES2) || defined(HAVE_COGL_GL)
+      else
+        {
+          if (blend_factor_uses_constant (blend_state->blend_src_factor_rgb) ||
+              blend_factor_uses_constant (blend_state
+                                          ->blend_src_factor_alpha) ||
+              blend_factor_uses_constant (blend_state->blend_dst_factor_rgb) ||
+              blend_factor_uses_constant (blend_state->blend_dst_factor_alpha))
+            {
+              float red =
+                cogl_color_get_red_float (&blend_state->blend_constant);
+              float green =
+                cogl_color_get_green_float (&blend_state->blend_constant);
+              float blue =
+                cogl_color_get_blue_float (&blend_state->blend_constant);
+              float alpha =
+                cogl_color_get_alpha_float (&blend_state->blend_constant);
 
 
-          GE (glBlendColor (red, green, blue, alpha));
+              GE (ctx, glBlendColor (red, green, blue, alpha));
+            }
+
+          if (ctx->glBlendEquationSeparate &&
+              blend_state->blend_equation_rgb !=
+              blend_state->blend_equation_alpha)
+            GE (ctx,
+                glBlendEquationSeparate (blend_state->blend_equation_rgb,
+                                         blend_state->blend_equation_alpha));
+          else
+            GE (ctx, glBlendEquation (blend_state->blend_equation_rgb));
+
+          if (ctx->glBlendFuncSeparate &&
+              (blend_state->blend_src_factor_rgb !=
+               blend_state->blend_src_factor_alpha ||
+               (blend_state->blend_src_factor_rgb !=
+                blend_state->blend_src_factor_alpha)))
+            GE (ctx, glBlendFuncSeparate (blend_state->blend_src_factor_rgb,
+                                          blend_state->blend_dst_factor_rgb,
+                                          blend_state->blend_src_factor_alpha,
+                                          blend_state->blend_dst_factor_alpha));
+          else
+            GE (ctx, glBlendFunc (blend_state->blend_src_factor_rgb,
+                                  blend_state->blend_dst_factor_rgb));
+        }
+#endif
+    }
+
+#if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES)
+
+  if (ctx->driver != COGL_DRIVER_GLES2)
+    {
+      /* Under GLES2 the alpha function is implemented as part of the
+         fragment shader */
+      if (pipelines_difference & (COGL_PIPELINE_STATE_ALPHA_FUNC |
+                                  COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE))
+        {
+          CoglPipeline *authority =
+            _cogl_pipeline_get_authority (pipeline,
+                                          COGL_PIPELINE_STATE_ALPHA_FUNC);
+          CoglPipelineAlphaFuncState *alpha_state =
+            &authority->big_state->alpha_state;
+
+          /* NB: Currently the Cogl defines are compatible with the GL ones: */
+          GE (ctx, glAlphaFunc (alpha_state->alpha_func,
+                                alpha_state->alpha_func_reference));
         }
 
-      if (have_blend_equation_seperate &&
-          blend_state->blend_equation_rgb != blend_state->blend_equation_alpha)
-        GE (glBlendEquationSeparate (blend_state->blend_equation_rgb,
-                                     blend_state->blend_equation_alpha));
-      else
-        GE (glBlendEquation (blend_state->blend_equation_rgb));
+      /* Under GLES2 the lighting parameters are implemented as uniforms
+         in the progend */
+      if (pipelines_difference & COGL_PIPELINE_STATE_LIGHTING)
+        {
+          CoglPipeline *authority =
+            _cogl_pipeline_get_authority (pipeline,
+                                          COGL_PIPELINE_STATE_LIGHTING);
+          CoglPipelineLightingState *lighting_state =
+            &authority->big_state->lighting_state;
+
+          GE (ctx, glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,
+                                 lighting_state->ambient));
+          GE (ctx, glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,
+                                 lighting_state->diffuse));
+          GE (ctx, glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,
+                                 lighting_state->specular));
+          GE (ctx, glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,
+                                 lighting_state->emission));
+          GE (ctx, glMaterialfv (GL_FRONT_AND_BACK, GL_SHININESS,
+                                 &lighting_state->shininess));
+        }
+    }
 
-      if (have_blend_func_separate &&
-          (blend_state->blend_src_factor_rgb != blend_state->blend_src_factor_alpha ||
-           (blend_state->blend_src_factor_rgb !=
-            blend_state->blend_src_factor_alpha)))
-        GE (glBlendFuncSeparate (blend_state->blend_src_factor_rgb,
-                                 blend_state->blend_dst_factor_rgb,
-                                 blend_state->blend_src_factor_alpha,
-                                 blend_state->blend_dst_factor_alpha));
-      else
 #endif
-        GE (glBlendFunc (blend_state->blend_src_factor_rgb,
-                         blend_state->blend_dst_factor_rgb));
-    }
-
-#ifndef HAVE_COGL_GLES2
-
-  /* Under GLES2 the alpha function is implemented as part of the
-     fragment shader */
-  if (pipelines_difference & (COGL_PIPELINE_STATE_ALPHA_FUNC |
-                              COGL_PIPELINE_STATE_ALPHA_FUNC_REFERENCE))
-    {
-      CoglPipeline *authority =
-        _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_ALPHA_FUNC);
-      CoglPipelineAlphaFuncState *alpha_state =
-        &authority->big_state->alpha_state;
-
-      /* NB: Currently the Cogl defines are compatible with the GL ones: */
-      GE (glAlphaFunc (alpha_state->alpha_func,
-                       alpha_state->alpha_func_reference));
-    }
-
-  /* Under GLES2 the lighting parameters are implemented as uniforms
-     in the progend */
-  if (pipelines_difference & COGL_PIPELINE_STATE_LIGHTING)
-    {
-      CoglPipeline *authority =
-        _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_LIGHTING);
-      CoglPipelineLightingState *lighting_state =
-        &authority->big_state->lighting_state;
-
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,
-                        lighting_state->ambient));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,
-                        lighting_state->diffuse));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,
-                        lighting_state->specular));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,
-                        lighting_state->emission));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_SHININESS,
-                        &lighting_state->shininess));
-    }
-
-#endif /* HAVE_COGL_GLES2 */
 
   if (pipelines_difference & COGL_PIPELINE_STATE_DEPTH)
     {
@@ -615,24 +575,42 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
         {
           if (ctx->depth_test_enabled_cache != TRUE)
             {
-              GE (glEnable (GL_DEPTH_TEST));
+              GE (ctx, glEnable (GL_DEPTH_TEST));
               ctx->depth_test_enabled_cache = depth_state->test_enabled;
             }
           flush_depth_state (depth_state);
         }
       else if (ctx->depth_test_enabled_cache != FALSE)
         {
-          GE (glDisable (GL_DEPTH_TEST));
+          GE (ctx, glDisable (GL_DEPTH_TEST));
           ctx->depth_test_enabled_cache = depth_state->test_enabled;
         }
     }
 
+  if (pipelines_difference & COGL_PIPELINE_STATE_LOGIC_OPS)
+    {
+      CoglPipeline *authority =
+        _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_LOGIC_OPS);
+      CoglPipelineLogicOpsState *logic_ops_state = &authority->big_state->logic_ops_state;
+      CoglColorMask color_mask = logic_ops_state->color_mask;
+      CoglFramebuffer *draw_framebuffer = cogl_get_draw_framebuffer ();
+
+      if (draw_framebuffer)
+        color_mask &= draw_framebuffer->color_mask;
+
+      GE (ctx, glColorMask (!!(color_mask & COGL_COLOR_MASK_RED),
+                            !!(color_mask & COGL_COLOR_MASK_GREEN),
+                            !!(color_mask & COGL_COLOR_MASK_BLUE),
+                            !!(color_mask & COGL_COLOR_MASK_ALPHA)));
+      ctx->current_gl_color_mask = color_mask;
+    }
+
   if (pipeline->real_blend_enable != ctx->gl_blend_enable_cache)
     {
       if (pipeline->real_blend_enable)
-        GE (glEnable (GL_BLEND));
+        GE (ctx, glEnable (GL_BLEND));
       else
-        GE (glDisable (GL_BLEND));
+        GE (ctx, glDisable (GL_BLEND));
       /* XXX: we shouldn't update any other blend state if blending
        * is disabled! */
       ctx->gl_blend_enable_cache = pipeline->real_blend_enable;
@@ -651,45 +629,54 @@ get_max_activateable_texture_units (void)
       int i;
 
 #ifdef HAVE_COGL_GL
-      /* GL_MAX_TEXTURE_COORDS is provided for both GLSL and ARBfp. It
-         defines the number of texture coordinates that can be
-         uploaded (but doesn't necessarily relate to how many texture
-         images can be sampled) */
-      if (cogl_features_available (COGL_FEATURE_SHADERS_GLSL) ||
-          cogl_features_available (COGL_FEATURE_SHADERS_ARBFP))
-        /* Previously this code subtracted the value by one but there
-           was no explanation for why it did this and it doesn't seem
-           to make sense so it has been removed */
-        GE (glGetIntegerv (GL_MAX_TEXTURE_COORDS, values + n_values++));
+      if (ctx->driver == COGL_DRIVER_GL)
+        {
+          /* GL_MAX_TEXTURE_COORDS is provided for both GLSL and ARBfp. It
+             defines the number of texture coordinates that can be
+             uploaded (but doesn't necessarily relate to how many texture
+             images can be sampled) */
+          if (cogl_features_available (COGL_FEATURE_SHADERS_GLSL) ||
+              cogl_features_available (COGL_FEATURE_SHADERS_ARBFP))
+            /* Previously this code subtracted the value by one but there
+               was no explanation for why it did this and it doesn't seem
+               to make sense so it has been removed */
+            GE (ctx, glGetIntegerv (GL_MAX_TEXTURE_COORDS,
+                                    values + n_values++));
 
-      /* GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS is defined for GLSL but
-         not ARBfp */
-      if (cogl_features_available (COGL_FEATURE_SHADERS_GLSL))
-        GE (glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,
-                           values + n_values++));
+          /* GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS is defined for GLSL but
+             not ARBfp */
+          if (cogl_features_available (COGL_FEATURE_SHADERS_GLSL))
+            GE (ctx, glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,
+                                    values + n_values++));
+        }
 #endif /* HAVE_COGL_GL */
 
 #ifdef HAVE_COGL_GLES2
+      if (ctx->driver == COGL_DRIVER_GLES2)
+        {
+          GE (ctx, glGetIntegerv (GL_MAX_VERTEX_ATTRIBS, values + n_values));
+          /* Two of the vertex attribs need to be used for the position
+             and color */
+          values[n_values++] -= 2;
 
-      GE (glGetIntegerv (GL_MAX_VERTEX_ATTRIBS, values + n_values));
-      /* Two of the vertex attribs need to be used for the position
-         and color */
-      values[n_values++] -= 2;
+          GE (ctx, glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,
+                                  values + n_values++));
+        }
+#endif
 
-      GE (glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,
-                         values + n_values++));
+#if defined (HAVE_COGL_GL) || defined (HAVE_COGL_GLES) /* not GLES2 */
+      if (ctx->driver != COGL_DRIVER_GLES2)
+        {
+          /* GL_MAX_TEXTURE_UNITS defines the number of units that are
+             usable from the fixed function pipeline, therefore it isn't
+             available in GLES2. These are also tied to the number of
+             texture coordinates that can be uploaded so it should be less
+             than that available from the shader extensions */
+          GE (ctx, glGetIntegerv (GL_MAX_TEXTURE_UNITS,
+                                  values + n_values++));
 
-#else /* HAVE_COGL_GLES2 */
-
-      /* GL_MAX_TEXTURE_UNITS defines the number of units that are
-         usable from the fixed function pipeline, therefore it isn't
-         available in GLES2. These are also tied to the number of
-         texture coordinates that can be uploaded so it should be less
-         than that available from the shader extensions */
-      GE (glGetIntegerv (GL_MAX_TEXTURE_UNITS,
-                         values + n_values++));
-
-#endif /* HAVE_COGL_GLES2 */
+        }
+#endif
 
       g_assert (n_values <= G_N_ELEMENTS (values) &&
                 n_values > 0);
@@ -786,7 +773,7 @@ flush_layers_common_gl_state_cb (CoglPipelineLayer *layer, void *user_data)
           if (unit_index == 1)
             unit->dirty_gl_texture = TRUE;
           else
-            GE (glBindTexture (gl_target, gl_texture));
+            GE (ctx, glBindTexture (gl_target, gl_texture));
           unit->gl_texture = gl_texture;
           unit->gl_target = gl_target;
         }
@@ -802,8 +789,9 @@ flush_layers_common_gl_state_cb (CoglPipelineLayer *layer, void *user_data)
 
   /* Under GLES2 the fragment shader will use gl_PointCoord instead of
      replacing the texture coordinates */
-#ifndef HAVE_COGL_GLES2
-  if (layers_difference & COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS)
+#if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GL)
+  if (ctx->driver != COGL_DRIVER_GLES2 &&
+      (layers_difference & COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS))
     {
       CoglPipelineState change = COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS;
       CoglPipelineLayer *authority =
@@ -812,10 +800,10 @@ flush_layers_common_gl_state_cb (CoglPipelineLayer *layer, void *user_data)
 
       _cogl_set_active_texture_unit (unit_index);
 
-      GE (glTexEnvi (GL_POINT_SPRITE, GL_COORD_REPLACE,
-                     big_state->point_sprite_coords));
+      GE (ctx, glTexEnvi (GL_POINT_SPRITE, GL_COORD_REPLACE,
+                          big_state->point_sprite_coords));
     }
-#endif /* HAVE_COGL_GLES2 */
+#endif
 
   cogl_handle_ref (layer);
   if (unit->layer != COGL_INVALID_HANDLE)
@@ -1315,7 +1303,7 @@ done:
      of the program object so they could be overridden by any
      attribute changes in another program */
 #ifdef HAVE_COGL_GLES2
-  if (!skip_gl_color)
+  if (ctx->driver == COGL_DRIVER_GLES2 && !skip_gl_color)
     {
       int attribute;
       CoglPipeline *authority =
@@ -1323,7 +1311,8 @@ done:
 
       attribute = _cogl_pipeline_progend_glsl_get_color_attribute (pipeline);
       if (attribute != -1)
-        GE (glVertexAttrib4f (attribute,
+        GE (ctx,
+            glVertexAttrib4f (attribute,
                               cogl_color_get_red_float (&authority->color),
                               cogl_color_get_green_float (&authority->color),
                               cogl_color_get_blue_float (&authority->color),
@@ -1354,7 +1343,7 @@ done:
   if (cogl_pipeline_get_n_layers (pipeline) > 1 && unit1->dirty_gl_texture)
     {
       _cogl_set_active_texture_unit (1);
-      GE (glBindTexture (unit1->gl_target, unit1->gl_texture));
+      GE (ctx, glBindTexture (unit1->gl_target, unit1->gl_texture));
       unit1->dirty_gl_texture = FALSE;
     }
 
